package net.krakatoaapi;

import de.dytanic.cloudnet.driver.CloudNetDriver;
import earth.krakatao.KraSocketClient;
import earth.krakatao.KraSocketClientConfig;
import earth.krakatao.events.KraSocketClientEventInitiater;
import earth.krakatao.events.KraSocketClientEventInterface;
import earth.krakatao.protocol.KraSocketClientProtocol;
import earth.krakatao.protocol.KraSocketClientProtocolDest;
import earth.krakatao.protocol.KraSocketClientProtocolMessage;
import earth.krakatao.protocol.KraSocketClientProtocolStatus;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.List;
import lombok.Getter;
import net.krakatoaapi.config.ConfigHandler;
import net.krakatoaapi.listener.PlayerJoinListener;
import net.krakatoaapi.listener.PlayerQuitListener;
import net.krakatoaapi.listener.SocketClientMessageListener;
import net.krakatoaapi.mongo.MongoManager;
import org.bukkit.Bukkit;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;

@Getter
public class KrakatoaAPI extends JavaPlugin {

  @Getter
  private static KrakatoaAPI instance;

  private ConfigHandler configHandler;
  private MongoManager mongoManager;
  private KraSocketClientProtocol kraSocketClientProtocol;
  private KraSocketClient kraSocketClient;
  private KraSocketClientEventInitiater kraSocketClientEventInitiater;
  private KraSocketClientEventInterface kraSocketClientEventInterface;

  @Override
  public void onEnable() {
    super.onEnable();

    instance = this;
    this.configHandler = new ConfigHandler();

    try {
      this.configHandler.load();
    } catch (IOException | InvalidConfigurationException e) {
      e.printStackTrace();
    }

    this.kraSocketClientProtocol = new KraSocketClientProtocol();

    KraSocketClientConfig kraSocketClientConfig = new KraSocketClientConfig(
        this.configHandler.getWebSocketProtocol(), this.configHandler.getWebSocketHost(),
        this.configHandler.getWebSocketPort(), this.configHandler.getWebSocketAccessKey(),
        CloudNetDriver.getInstance().getComponentName());

    this.kraSocketClientEventInitiater = new KraSocketClientEventInitiater();
    this.kraSocketClientEventInterface = new SocketClientMessageListener();

    try {
      this.kraSocketClient = new KraSocketClient(kraSocketClientConfig,
          this.kraSocketClientEventInterface, this.kraSocketClientProtocol);

      this.kraSocketClient.getSocketClient().connect();
    } catch (URISyntaxException e) {
      e.printStackTrace();
    }

    this.mongoManager = new MongoManager(this.configHandler.getMongodbHost(),
        this.configHandler.getMongodbPort(), this.configHandler.getMongodbUsername(),
        this.configHandler.getMongodbPassword());
    this.mongoManager.connect(this.configHandler.getMongodbDatabase());

    loadListeners();

    // call to proxy that the server is only for the auto restart system
    Bukkit.getScheduler().runTaskLaterAsynchronously(this, () -> {
      KraSocketClientProtocolMessage kraSocketClientProtocolMessage = new KraSocketClientProtocolMessage(
          KraSocketClientProtocolStatus.SEND.getStatus(), 0,
          KraSocketClientProtocolDest.PROXY.getStatus(), "c7b023ac03034275923d3a1028e887a0",
          (short) 59739,
          CloudNetDriver.getInstance().getComponentName());

      this.kraSocketClient.getSocketClient().SendMessage(kraSocketClientProtocolMessage);
    }, 1000);
  }

  @Override
  public void onDisable() {
    super.onDisable();

    this.kraSocketClient.getSocketClient().getConnection().close();
    this.kraSocketClient.getSocketClient().getConnection().closeConnection(1, "");

    instance = null;
  }

  public void loadListeners() {
    PluginManager pluginManager = Bukkit.getPluginManager();
    List.of(new PlayerJoinListener(), new PlayerQuitListener()).forEach(listener -> {
      pluginManager.registerEvents(listener, this);
    });
  }
}
